#! /usr/bin/python3

import argparse

import os
import testeur
import pandas
import json
import re

# Configuration
default_host = "localhost"
default_user = "ubuntu"

dockers = {}

nbOK = 0
nbNOK = 0

def ok(msg):
    global nbOK
    
    nbOK=nbOK+1
    print ("\033[1;32m{}\033[0m\t{}".format("[+]", msg))

def nok(msg):
    global nbNOK
    
    nbNOK=nbNOK+1

    #print ("\033[48;5;196m\033[1m\33[5m{}\033[0m\t{}".format("Not OK", msg))
    print ("\033[1;31m{}\033[0m\t{}".format("[-]", msg))

def traite_ok_temp(vok, mess):
    if (vok == 1):
        ok(mess)
    else:
        nok(mess)

def affiche_results(results):
    for result in results:
        traite_ok_temp(result[0], result[1])

def get_descript(t, iddocker):
    """
    Récupère une descrition d'un docker dont on a le nom a priori a partir
    de la commande docker inspect
    """
    sortie, err = t.test("sudo docker inspect {}".format(iddocker))
    if len(err) != 0:
        raise ValueError("Le docker {} n'existe pas".format(iddocker))
    descr = {}
    inspect = json.loads("".join(sortie))[0]

    descr['Name'] = inspect['Name']
    descr['HostnamePath'] = inspect['HostnamePath']
    descr['HostsPath'] = inspect['HostsPath']

    descr['Volumes'] = inspect['Mounts']
    descr['Ports'] = inspect['NetworkSettings']['Ports']
    descr['Networks'] = {}
    for net in inspect['NetworkSettings']['Networks'].keys():
        descr['Networks'][net] = {}
        descr['Networks'][net]['IPAddress'] = inspect['NetworkSettings']['Networks'][net]['IPAddress']
        descr['Networks'][net]['Prefix'] = inspect['NetworkSettings']['Networks'][net]['IPPrefixLen']
        descr['Networks'][net]['Aliases'] = inspect['NetworkSettings']['Networks'][net]['Aliases']

    descr['Env'] = inspect['Config']['Env']
    descr['ExtraHosts'] = inspect['HostConfig']['ExtraHosts']
    return descr

def change_status_docker(nom, status, t):
    r,e = t.test("docker {} {}".format(status, nom))
    if (len(e) != 0):
        print("La commande docker {} {} a occasionner l'erreur {}".format(status, nom, '\n'.join(e)))


def existance_fichiers(t, fichiers):
    manque = []
    for fich in fichiers:
        if not os.path.exists(fich):
            manque.append(fich)
    return manque

def test_install_docker(t):
    """
    test l'installation de docker, si le docker démon fonctionne et si
    le proxy est bien mis
    """
    sortie, err = t.test("sudo docker ps -a")
    if len(err) != 0:
        print("Il y a une erreur pour la commande docker ps")
        for l in err:
            print("\t\"{}\"".format(l))
        nok("Docker a un problème")
    else:
        ok("Docker semble installé et fonctionnel")



def test_presence_docker(t, dat):
    """
    teste si les dockers décrit par dat existent
    
    effet de bords : stocke la description des dockers utilisés dans un tableau et lance eventuellement une commande pour allumer les docker

    """
    sortie, err = t.test("sudo docker ps -a", result="file")
    sort = pandas.read_fwf(sortie,header=0)

    
    print("Liste des Dockers trouvés :\n");
    for l in sort['NAMES']:
        print("\t\"{}\"".format(l))
    print(79 * "-")
    dockers = {}
    bon = True
    for regexnom in dat.keys():
        # le docker est-il dans la liste des noms
        trouv = sort[sort['NAMES'].map(lambda x: bool(re.match(regexnom,x)))]
        if len(trouv) == 1:
            pos = trouv.index[0]
            nom = str(trouv.loc[pos,'NAMES'])
            ok("Docker \"{}\" trouvé en position {}".format(nom, pos))
            if (dat[regexnom]['changeStatus'] is not None):
                change_status_docker(nom, dat[regexnom]['changeStatus'], t)
            dockers[dat[regexnom]['service']] = get_descript(t, nom)
            if re.match("^{}.*$".format(dat[regexnom]['image']),sort['IMAGE'][pos]):
                 ok("Le nom de l'image est bien \"{}\"".format(sort['IMAGE'][pos]))
            else:
                 bon = False
                 nok("Le nom de l'image est \"{}\" mais devrait être \"{}\""
                     .format(sort['IMAGE'][pos], dat[regexnom]['image']))
        elif len(trouv) > 1:
            bon = False
            nok("\"{}\" dockers trouvés pour la regex {}".format(len(trouv), regexnom))
            nok("Donc l'image n'est pas bonne non plus")
        else:
            bon = False
            nok("Docker \"{}\" non trouvé".format(regexnom))
            nok("Donc l'image n'est pas bonne non plus")

    return dockers


def verif_partage(vol, source, dest, moderw):
    return ((re.match(source,vol['Source']))
            and (re.match(dest,vol['Destination']))
            and (vol['RW'] == moderw))


def verif_venv(Venvs, valeur, messageOK, messageNOK):
    try:
        if (any(map(lambda x: re.match(valeur, x), Venvs))):
            ok(messageOK)
            return True
    except:
        print("{} n'est pas iterable".format(Venvs))
    
    nok(messageNOK)
    return False
        
def verif_ip(docker, reseau, ip):
    if reseau in docker['Networks'].keys():
        if (docker['Networks'][reseau]['IPAddress'] == ip):
            ok("{} a bien l'adresse {} dans le réseau {}"
               .format(docker['Name'], ip, reseau)
            )
            return True
        else:
            nok("{} a l'adresse {} au lieu de {} dans le réseau {}"
               .format(docker['Name'],
                       docker['Networks'][reseau]['IPAddress'], ip, reseau))
            return False
    else:
        nok("{} n'a pas d'adresse dans le réseau {}"
            .format(docker['Name'], reseau))
        return False

def verif_alias(descr, reseau, alias):
    """
    Vérifie la présence d'alias réseau
    
    @param descr : la description où chercher
    @param reseau, alias : le réseau et l'alias à chercher
    @return : un résultat de rechercher (nbpoint, message)
    """
    if not (reseau in descr['Networks'].keys()):
        return (0, "le réseau {} n'est pas dans {}".format(reseau, descr['Name']))    
    if (alias in descr['Networks'][reseau]['Aliases']):
        return (1, "le docker {} a bien l'alias {} est dans le réseau {}".format(descr['Name'], alias, reseau))
    else:
        return (0, "le docker {} n'a pas l'alias {} dans le réseau {}".format(descr['Name'], alias, reseau))

def test_docktest(docker, t):
    nom = "mysql";
    passwd = "passroot";
    result, err = t.test("sudo cat '{}'".format(docker['HostnamePath']))
    if ((len(result)>0) and (result[0]=='{}\n'.format(nom))) :
        ok("Le hostname du docker est bien test")
    else:
        if (len(result)>0):
            nok("Le hostname du docker est {} au lieu de {}".format(result[0], nom))
        else:
            nok("Le hostname du docker n'existe pas")

    if (any(map(lambda x: x=="MYSQL_ROOT_PASSWORD={}".format(passwd), docker['Env']))):
        ok("on trouve la variable d'env pour le password de root")
    else:
        nok("Le mot de passe root ne semble pas bon")

    if (any(map(lambda x:
                verif_partage(x,
                              "/home/ubuntu/docker/datatest",
                              "/var/lib/mysql",
                              True
                ),
                docker['Volumes']))):
        ok("Le partage de données est trouvé")
    else:
        nok("Pas de partage pour les données")


    res, err = t.test('ls /home/ubuntu/docker/datatest/')
    if (any(map(lambda x: x=='BASE_A\n', res))):
        ok("On trouve la base BASE_A")
    else:
        print('les fichiers du partage "/home/ubuntu/docker/datatest/" sont ')
        for f in res:
            print ("  - {}".format(f))
        nok("On ne trouve la base BASE_A")

    res, err = t.test('ls /home/ubuntu/docker/datatest/')
    if (any(map(lambda x: x=='BASE_B\n', res))):
        ok("On trouve la base BASE_B")
    else:
        print('les fichiers du partage "/home/ubuntu/docker/datatest/" sont ')
        for f in res:
            print ("  - {}".format(f))
        nok("On ne trouve la base BASE_B")

    return True

    
def test_dockbdd(docker, t):
    nom = "basededonnee"
    result, err = t.test("sudo cat '{}'".format(docker['HostnamePath']))
    if ((len(result)>0) and (result[0]=='{}\n'.format(nom))) :
        ok("Le hostname du docker est bien test")
    else:
        if (len(result)>0):
            nok("Le hostname du docker est {} au lieu de {}".format(result[0], nom))
        else:
            nok("Le hostname du docker n'existe pas")

    #print("Vérification des variables d'environement")
    venvs = [
        ["MYSQL_ROOT_PASSWORD=passroot", "On trouve le mot de passe root", "Le mot de passe root n'est pas bon"],
     #  ["MYSQL_USER=nextcloud","On trouve l'utilisateur nextcloud", "On ne trouve pas l'utilisateur nextcloud"],
     #  ["MYSQL_PASSWORD=passnc","MdP de nextcloud", "mauvais MdP pour nextcloud"],
     #  ["MYSQL_DATABASE=nextcloud","La base de donnée", "Pas la base de donnée"]
    ]
    for ve in venvs:
        verif_venv(docker["Env"], ve[0], ve[1], ve[2])

    if (any(map(lambda x:
                verif_partage(x,
                              "/home/ubuntu/docker/data",
                              "/var/lib/mysql",
                              True
                ),
                docker['Volumes']))):
        ok("Le partage de données est trouvé")
    else:
        nok("Pas de partage pour les données")

#    res, err = t.test('ls /home/ubuntu/docker/data/')
#    if (any(map(lambda x: x=='nextcloud\n', res))):
#        ok("On trouve la base nextcloud")
#    else:
#        print('les fichiers du partage "/home/ubuntu/docker/datatest/" sont ')
#        for f in res:
#            print ("  - {}".format(f))
#        nok("On ne trouve la base nextcloud")

    verif_ip(docker, "interne", "172.18.100.10")
        
    return True
    
def test_dockpma(docker, t):

    #print("Vérification des variables d'environement")
    venvs = [
        ["PMA_HOST=dockbase", "La base est bien configurée", "La base de donnée n'est pas bien configurée le serveur doit être nomé 'dockbase'"],
        ["PMA_ABSOLUTE_URI=(http://192.168.[0-9]+.[0-9]+)?/phpMyAdmin/?","L'url d'accès est bonne pour nginx", "L'url d'accès n'est pas bonne pour nginx"],
    ]
    for ve in venvs:
        verif_venv(docker["Env"], ve[0], ve[1], ve[2])

    verif_ip(docker, "interne", "172.18.100.11")
    # verif_venv(docker["ExtraHosts"], "bdd:172.18.100.10",
    #            "La liaison avec la bdd est bonne",
    #            "La liaison avec la bdd n'est pas bonne"
    # )

    res, err = t.test("wget --no-proxy http://172.18.100.11 -O -")
    if (any(map(lambda x: re.search("HTTP request sent, awaiting response... 200 OK",x), err))):
        ok("dockMyAdm semble répondre")
    else:
        nok("dockMyAdm ne semble pas répondre")

    bon = False
    for lig in res:
        #m = re.search("PMA_commonParams", lig)
        m = re.search("PMA_", lig)
        if (m):
            ok("dockMyAdm semble être un docker phpMyAdmin")
            m1 = re.search('rootPath:"([^"]*)"', lig)
            if (re.match("/?phpMyAdmin/?",m1.group(1))):
                ok("le rootpath est bien {}".format(m1.group(1)))
            else:
                nok("le rootpath est {}".format(m1.group(1)))
            bon=True
    if (not bon):
        nok("dockMyAdm ne semble pas être un docker phpMyAdmin")
        
        
    

def test_docknginx(docker, t):

    verif_ip(docker, "interne", "172.18.100.15")
    # verif_venv(docker["ExtraHosts"], "(dock)?pma:172.18.100.11",
    #            "La liaison avec pma est bonne",
    #            "La liaison avec pma n'est pas bonne (la machine doit s'appeléer pma ou dockpma"
    # )
    
    if (any(map(lambda x:
                verif_partage(x,
                              "/home/ubuntu/docker/nginx\.conf",
                              "/etc/nginx/nginx\.conf",
                              False
                ),
                docker['Volumes']))):
        ok("Le partage de configuration /home/ubuntu/docker/nginx.conf vers /etc/nginx/nginx.conf est trouvé en RO")
    else:
        nok("Pas de partage de configuration (nginx.conf)")

    if (any(map(lambda x:
                verif_partage(x,
                              "/home/ubuntu/docker/nginx",
                              "/www",
                              True
                ),
                docker['Volumes']))):
        ok("Le partage de site /home/ubuntu/docker/nginx vers /www est trouvé en RW")
    else:
        nok("Pas de partage de site (/www)")

    res, err = t.test("wget --no-proxy http://localhost/phpMyAdmin/ -O -")
    if (any(map(lambda x: re.search("HTTP request sent, awaiting response... 200 OK",x), err))):
        ok("pma semble répondre via nginx qui semble répondre")
    else:
        nok("dockpma ne semble pas répondre via nginx")

    bon = False
    for lig in res:
        m = re.search("PMA_", lig)
        if (m):
            ok("dockpma semble être un docker phpMyAdmin")
            m1 = re.search('rootPath:"([^"]*)"', lig)
            if (re.match("/?phpMyAdmin/?",m1.group(1))):
                ok("le rootpath est bien {}".format(m1.group(1)))
            else:
                nok("le rootpath est {}".format(m1.group(1)))
            bon=True
    if (not bon):
        nok("dockpma ne semble pas être un docker phpMyAdmin")

def nettoye_session_owc(t, dock):
    res,err = t.test("docker exec -t {} ls /mnt/data/sessions/".format(dock))
    if (len(err)>0):
        print("La commande ls a donné une erreur {}".format(' '.join(err)))
    for lig in res:
        lig = re.sub("\n", "", lig)
        lig = re.sub("\r", "", lig)
#        print("lig='{}'".format(lig))
        files = lig.split()
#        print(files)
        for file in files:
            if (file != ''):
                print("efface '{}'".format(file))
                com = "docker exec -t {} rm /mnt/data/sessions/{}".format(dock,file)
                res,err = t.test(com)
                if (len(err)>0):
                    print("La commande {} a donné une erreur {}".format(com, ' '.join(err)))




def verif_ping(t, dockfrom, to, expect):
    com = "docker exec -t {} ping -c 3 {}".format(dockfrom, to)
    res, err = t.test(com)
    if (len(err)>0):
        print("La commande {} stop à générée l'erreur {}"
              .format(com, " ".join(err)))
        return (0, "Impossible de faire un ping")
    address = None
    for lig in res:
        m = re.match(".*[0-9]+ bytes from ([0-9.]+): seq=[0-2] ttl=64 time=[0-9.]+ .*", lig)
        if (m):
            address = m.group(1)
            break
    if (address is None):
        return (0, "Aucun ping ne passe entre {} et {} cela donne {}".format(dockfrom, to, " ".join(res)))
    
    if (address == expect):
        return (1, "Un ping au moins fonctionne entre {} et {}".format(dockfrom, to))
    else:
        return (0, "Un ping au moins fonctionne entre {} et {} mais avec une mauvaise adresse".format(dockfrom, to))
        
def test_compose_aliases(t, descriptions):
    alias = [
        {'alias': "mysql", 'service': "mysql"},
        {'alias': "bdd",   'service': "mysql"},
        {'alias': "pma",   'service': "phpmyadmin"},
        {'alias': "owc",   'service': "owncloud"},
    ]

    if (('phpmyadmin' not in descriptions.keys())
        or ('compose_rescompose' not in descriptions['phpmyadmin']['Networks'].keys())):
        results = [(0, "impossible de tester l'alias '{}' car le docker phpmyadmin ou le réseau rescompose n'est pas trouvé".format(a['alias'])) for a in alias]
        affiche_results(results)
        return results

    results= []
    for a in alias:
        result = verif_ping(t,
                            descriptions['phpmyadmin']['Name'],
                            a['alias'],
                            descriptions[a['service']]['Networks']['compose_rescompose']['IPAddress'])
        traite_ok_temp(result[0], result[1])
        results.append(result)

    return results

                


        
def test_all(server="localhost", user=None, key=None, parts = ["debut"]):
    global nbOK
    global nbNOK
    
    nbOK=0
    nbNOK=0
    
    if server == "localhost":
        t = testeur.testeur()
    else:
        t = testeur.testeur_ssh(server, user, key=key)

    #    test_install_docker(t)

    if (('debut' in parts) or ('owncloud' in parts)):
        
        dat = {
            #'dockmysql'    : {'image': 'mysql:5.7.27'  , 'changeStatus':'stop'  , 'service': 'dockmysql'}, 
            'dockbase'    : {'image': 'mysql:5.7.27'  , 'changeStatus':'start' , 'service': 'dockbase'},
            'dockMyAdm'   : {'image': 'phpmyadmin'    , 'changeStatus':'start' , 'service': 'dockMyAdm'},
            'dockFront'   : {'image': 'nginx'         , 'changeStatus':'start' , 'service': 'dockFront'},
            #'dockNextcloud': {'image': 'nextcloud-perso', 'changeStatus':'start' , 'service': 'dockNextcloud'},
        }
    
        dockers = test_presence_docker(t, dat)
        #verifie_docker(t)

    if ('debut' in parts):
        
        #print("######################################")
        #print("## Vérification du docker dockmysql ##")
        #print("######################################")
        #if ('dockmysql' in dockers.keys()):
        #    test_docktest(dockers['dockmysql'], t)
        
        print("#####################################")
        print("## Vérification du docker dockbase ##")
        print("#####################################")
        if ('dockbase' in dockers.keys()):
            test_dockbdd(dockers['dockbase'], t)
        print("######################################")
        print("## Vérification du docker dockMyAdm ##")
        print("######################################")
        if ('dockMyAdm' in dockers.keys()):
            test_dockpma(dockers['dockMyAdm'], t)
        print("######################################")
        print("## Vérification du docker dockFront ##")
        print("######################################")
        if ('dockFront' in dockers.keys()):
            test_docknginx(dockers['dockFront'], t)

        
    t.close()
    return (nbOK,nbNOK)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Passe des scripts de test à distance ou en local")
    parser.add_argument("-s", "--server", type=str, dest="server", default=default_host, help="La machine sur laquelle se connecter")
    parser.add_argument("-u", "--user", type=str, dest="user",
                        default=default_user, help="Le nom de l'utilisateur pour se connecter sur la machine (n'est pas considéré pour la machine locale")
    parser.add_argument("-k", "--key", type=str, dest="key", default=None, help="nom du fichier de clef")

    #parser.add_argument("parts", type=str, metavar="part", nargs='+',
    #                    help="Parties de la vérification à faire 'debut', 'owncloud' ou 'compose'."
    #                        +"Il ne faut pas utiliser compose avec les autres")
    
    args = parser.parse_args()

    print(79 * "#")
    #print("Test du serveur {} pour {}".format(args.server, " ".join(args.parts)))
    print("Test du serveur {}".format(args.server))
    print(79 * "#")
    
    print("(This checking tool was originally wrote by Fabien Rico and then modified by Jp on purpose)")

    #nbok, nbnok = test_all(args.server, args.user,args.key, args.parts)
    nbok, nbnok = test_all(args.server, args.user,args.key)
    print(79 * "#")
    print ("## NOTE FINALE : {}/{}".format(nbOK, nbOK+nbNOK))
    print(79 * "#")
    
